from urllib.parse import quote
import re
import json
import copy
import random
import urllib.request
from .query_parser import QueryParser
from .config import ResourceConfig
from .diagnostics import Diagnostic, DiagnosticTypes


class LitteraeQueryParser(QueryParser):
    """
    Parses search queries for Formulae, Litterae, Chartae.
    """

    def build_get_string(self, getParams, config):
        """
        Build a GET string (everything after the ?) from a description
        of the GET parameters in the getParams list.
        """
        termIndexes = self.term_indexes(getParams)
        nWords = len(termIndexes)
        boolOperatorMentioned = False
        s = 'source=advanced&sort=urn&lemma_search=False'
        for param in getParams:
            nSfx = ''
            if param[1] > 0:
                nSfx = str(param[1])
            if param[0] == 'bool_operator':
                if boolOperatorMentioned:
                    continue
                boolOperatorMentioned = True
            s += '&' + param[0] + nSfx + '=' + quote(str(param[2]))
            if param[0] == 'q_':
                s += '&fuzziness_' + nSfx + '=0'
                s += '&slop_' + nSfx + '=0'
                s += '&in_order_' + nSfx + '=False'
                s += '&search_field_' + nSfx + '=text'
                s += '&exclude_q_' + nSfx + '='
        return s

    def term_query(self, query, config):
        """
        Return list of query parameters for one term or sequence of terms.
        """
        if len(query) >= 2 and query.startswith('"') and query.endswith('"'):
            query = query[1:len(query)-1]
        if len(query) <= 0:
            return Diagnostic(DiagnosticTypes.sru, 10)
        getParams = [['q_', 1, query]]
        return getParams

    def binary_bool(self, strOp, operandL, operandR, config):
        if len(operandL) <= 0 or len(operandR) <= 0:
            raise Diagnostic(DiagnosticTypes.sru, 10)
        termsL = self.term_indexes(operandL)
        operandR = self.shift_term_indexes(operandR, max(termsL))
        termsR = self.term_indexes(operandR)
        if strOp == 'AND':
            getParamsNew = operandL + operandR
            if any(p[0] == 'bool_operator' and p[2] == 'should' for p in getParamsNew):
                message = 'FLC web interface does not support queries that ' \
                          'combine AND and OR.'
                raise Diagnostic(DiagnosticTypes.sru, 48, message=message)
            return getParamsNew + [['bool_operator', -1, 'must']]
        elif strOp == 'OR':
            getParamsNew = operandL + operandR
            if any(p[0] == 'bool_operator' and p[2] == 'must' for p in getParamsNew):
                message = 'FLC web interface does not support queries that ' \
                          'combine AND and OR.'
                raise Diagnostic(DiagnosticTypes.sru, 48, message=message)
            return getParamsNew + [['bool_operator', -1, 'should']]
        raise Diagnostic(DiagnosticTypes.sru, 37, details=strOp)

    def not_bool(self, operand, config):
        # TODO: implement
        raise NotImplementedError()

    def send_query(self, strGetParams: str, config: ResourceConfig):
        """
        Send the translated query to the Litterae instance. Return JSON results
        returned by the corpus.
        """
        url = config.resource_base_url.strip('/') + '/results?' + strGetParams
        response = urllib.request.urlopen(url)
        data = response.read()
        encoding = response.info().get_content_charset('utf-8')
        responseHTML = data.decode(encoding)
        return responseHTML


if __name__ == '__main__':
    pass

