# Contains functions called by the top-level view functions
# that process the user's request and return a rendered XML
# template
from typing import Optional
from fastapi import Request, Query, Response
from .enums import *
from .diagnostics import Diagnostic
from .config import ResourceConfig


def fatal_response(operation: Operation,
                   version: SRUVersion,
                   diagnostics: list[Diagnostic],
                   request, templates):
    """
    Return a response with the fatal diagnostics
    and no other payload.
    """
    diagStr = [str(d) for d in diagnostics]
    if operation == Operation.explain:
        templateName = 'explain_response_2.0.xml'
        if version == SRUVersion.v1_2:
            templateName = 'explain_response_1.2.xml'
        return templates.TemplateResponse(templateName,
                                          {
                                              'request': request,
                                              'diagnostics': diagStr
                                          },
                                          media_type='application/xml')
    elif operation == Operation.searchRetrieve:
        return templates.TemplateResponse('search_retrieve_response.xml',
                                          {
                                              'request': request,
                                              'diagnostics': diagStr,
                                              'n_hits': 0,
                                          },
                                          media_type='application/xml')


def process_search_retrieve(version: SRUVersion,
                            queryType: QueryType,
                            query: str,
                            searchOptions: dict[str, str],
                            config: Optional[ResourceConfig],
                            diagnostics: list[Diagnostic],
                            app, request, templates):
    """
    Process a searchRetrieve request.
    Return a rendered XML response.
    """
    if config.platform == CorpPlatform.tsakorpus:
        try:
            strGetParams = app.qp_tsakorpus.translate_fcsql(query, config)
            print(strGetParams)
            res = app.qp_tsakorpus.send_query(strGetParams, config)
        except Diagnostic as diag:
            return fatal_response(Operation.searchRetrieve, version, [diag], request, templates)
        records, nHits, diagnostics = app.rp_tsakorpus.parse(res, config, searchOptions['x-fcs-dataviews'])
        records = [r.as_dict() for r in records]
        diagnostics = [str(d) for d in diagnostics]
        return templates.TemplateResponse('search_retrieve_response.xml',
                                          {
                                              'request': request,
                                              'n_hits': nHits,
                                              'records': records,
                                              'diagnostics': diagnostics
                                          },
                                          media_type='application/xml')
    elif config.platform == CorpPlatform.litterae:
        try:
            strGetParams = app.qp_litterae.translate_fcsql(query, config)
            print(strGetParams)
            # return strGetParams
            res = app.qp_litterae.send_query(strGetParams, config)
            print(res)
        except Diagnostic as diag:
            return fatal_response(Operation.searchRetrieve, version, [diag], request, templates)
        records, nHits, diagnostics = app.rp_litterae.parse(res, config, searchOptions['x-fcs-dataviews'])
        records = [r.as_dict() for r in records]
        diagnostics = [str(d) for d in diagnostics]
        return templates.TemplateResponse('search_retrieve_response.xml',
                                          {
                                              'request': request,
                                              'n_hits': nHits,
                                              'records': records,
                                              'diagnostics': diagnostics
                                          },
                                          media_type='application/xml')


def process_request(operation: Operation,
                    version: SRUVersion,
                    queryType: QueryType,
                    query: str,
                    searchOptions: dict[str, str],
                    config: Optional[ResourceConfig],
                    diagnostics: list[Diagnostic],
                    app, request, templates):
    """
    Process validated user request that came in through the endpoint()
    function in main.py.
    Return a rendered template.
    :param diagnostics: List of diagnostics produced by the validation
    function.
    """
    # If something is clearly wrong with the query, return
    # a response with the list of diagnostics
    if config is None or any(d.is_fatal() for d in diagnostics):
        return fatal_response(operation, version, diagnostics, request, templates)

    # If everything looks good, proceed to query parsing
    if operation == Operation.searchRetrieve:
        return process_search_retrieve(version, queryType, query, searchOptions, config, diagnostics, app, request, templates)


if __name__ == '__main__':
    pass
