from fastapi import FastAPI, Request, Query, Response
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse
from common.query_parser import QueryParser
from common.litterae_query_parser import LitteraeQueryParser
from common.litterae_response_parser import LitteraeResponseParser
from common.tsakorpus_query_parser import TsakorpusQueryParser
from common.tsakorpus_response_parser import TsakorpusResponseParser
from common.enums import *
from common.diagnostics import Diagnostic
from common.config import ResourceConfig, read_configs
from common.views_logic import *
import json
import os
import re
import copy
import uvicorn

app = FastAPI()
app.mount('/static', StaticFiles(directory='static'), name='static')
templates = Jinja2Templates(directory='static')

app.qp = QueryParser()

app.qp_litterae = LitteraeQueryParser()
app.rp_litterae = LitteraeResponseParser()
app.qp_tsakorpus = TsakorpusQueryParser()
app.rp_tsakorpus = TsakorpusResponseParser()
app.configs = read_configs()


@app.get('/')
def root():
    return {"message": "Hello World"}


@app.get('/fcs-endpoint/{corpusID}')
def endpoint(
        request: Request,
        corpusID: str,
        operation: Operation = Operation.explain,
        version: SRUVersion = SRUVersion.v2_0,
        queryType: QueryType = QueryType.fcs,
        query: str = '',
        xFcsEndpointDescription: str = Query(
            default='',
            alias='x-fcs-endpoint-description'
        ),
        xFcsContext: str = Query(
            default='',
            alias='x-fcs-context'
        ),
        xFcsDataviews: str = Query(
            default='',
            alias='x-fcs-dataviews'
        ),
        xFcsRewritesAllowed: str = Query(
            default='',
            alias='x-fcs-rewrites-allowed'
        )
        ):
    searchOptions = {
        'x-fcs-endpoint-description': xFcsEndpointDescription,
        'x-fcs-context': xFcsContext,
        'x-fcs-dataviews': xFcsDataviews,
        'x-fcs-rewrites-allowed': xFcsRewritesAllowed
    }

    # Check if the corpus ID is correct
    if corpusID not in app.configs:
        message = 'No corpus with this ID (' + corpusID +') is served by this Endpoint. ' \
                  'Valid corpus IDs are: ' + '; '.join(cID for cID in sorted(app.configs)) + '.'
        diag = Diagnostic(DiagnosticTypes.sru, 235, message=message)  # "Database does not exist"
        return process_request(operation, version, queryType, query, searchOptions, None, [diag], app, request, templates)
    config = app.configs[corpusID]

    # Check for common problems with parameter values
    diagnostics = app.qp.validate_query(operation, version, queryType, query,
                                        xFcsEndpointDescription, xFcsContext,
                                        xFcsDataviews, xFcsRewritesAllowed)
    # Now, do the substantial things
    return process_request(operation, version, queryType, query, searchOptions, config, diagnostics, app, request, templates)
    # return {'operation': operation, 'version': version}


if __name__ == '__main__':
    uvicorn.run(
        'main:app',
        host='0.0.0.0',
        port=5000,
        reload=True
    )
